// *********************************
// **                             **
// **   Vulcan.Net GDI+ demo!     **
// **   Written by Chris Pyrgas   **
// **   November 2005             **
// **                             **
// **   First presented at        **
// **   VODC 2005                 **
// **                             **
// **   Enjoy!                    **
// **                             **
// *********************************


REFERENCES "System.windows.forms"
REFERENCES "System.Drawing"
REFERENCES "Microsoft.VisualBasic"

USING System.Windows.Forms
USING System.Drawing
USING System.Drawing.Drawing2D

FUNCTION Start() AS VOID

	LOCAL oDemoForm AS Form
	LOCAL oDemoPictureBox AS DemoPictureBox
	
	oDemoPictureBox:=DemoPictureBox{}
	oDemoPictureBox:Dock:=DockStyle.Fill
	
	oDemoForm:=Form{}
	oDemoForm:ClientSize:=Size{800,600}
	oDemoForm:Location:=Point{100,50}
	oDemoForm:Text:='VODC GDI+ Demo'
	oDemoForm:MinimizeBox:=FALSE
	oDemoForm:MaximizeBox:=FALSE
	oDemoForm:StartPosition := FormStartPosition.Manual
	oDemoForm:FormBorderStyle:=FormBorderStyle.FixedDialog
	oDemoForm:Controls:Add(oDemoPictureBox)
	oDemoForm:KeyDown += KeyEventHandler{oDemoPictureBox,@DemoPictureBox.FormKeyDown}
	oDemoForm:ShowDialog()

RETURN

CLASS DemoPictureBox INHERIT PictureBox

	PROTECT nCounter AS INT
	PROTECT oTimer AS Timer
	PROTECT oPen AS Pen
	PROTECT oFont AS Font
	PROTECT oWaveFont AS Font
	PROTECT nBoxes,nLines AS INT
	
	PROTECT DIM aBoxes[100] AS Box
	PROTECT DIM aLines[100] AS Line
	
	PROTECT oBitmap AS Bitmap
	PROTECT lF1Pressed AS LOGIC

CONSTRUCTOR() CLASS DemoPictureBox

	LOCAL n AS INT

	SUPER()
	
	SELF:oPen:=Pen{Color.Red}
	
	FOR n:=1 UPTO 100
		SELF:aBoxes[n]:=Box{}
		SELF:aLines[n]:=Line{n}
	NEXT
	
	SELF:oBitmap := Bitmap{'Image.jpg'}
	
	SELF:BackColor:=Color.White
	SELF:BackColor:=Color.Black
	
	SELF:oFont:=Font{'Arial',60.0}
	SELF:oWaveFont:=Font{'Arial',68.0}
	
	oTimer:=Timer{}
	oTimer:Tick+=EventHandler{SELF,@DemoPictureBox.OnTimerTick}
	oTimer:Interval:=15
	oTimer:Start()

RETURN

METHOD FormKeyDown(o AS OBJECT,e AS KeyEventArgs) AS VOID CLASS DemoPictureBox
	IF e:KeyCode == Keys.F1 .and. SELF:nCounter > 3300
		SELF:lF1Pressed := TRUE
	END IF
RETURN


METHOD OnTimerTick(o AS OBJECT,e AS EventArgs) AS VOID CLASS DemoPictureBox

	LOCAL n AS INT

	SELF:nCounter++
//	SELF:Parent:Text:=Str(nCounter)
	
	FOR n:=1 UPTO 100
		SELF:aLines[n]:Play(SELF:nCounter)
	NEXT
	
	DO CASE
	CASE nCounter<400
		nBoxes:=0
	CASE nCounter<10000
		nBoxes:=((nCounter-400) /60) * ((nCounter-400) /60) / 5 + 1
		nBoxes:=Min(nBoxes,100)
		IF nCounter>1500
			nBoxes:=100-(nCounter-1500) / 4
		END IF
		nBoxes:=Max(nBoxes,0)
	END CASE
	
	IF nCounter>1700
		nLines:=Min((nCounter-1700)/5, 70)
	END IF
	IF nCounter>3800
		nLines:=0
		nBoxes:=0
	END IF
	
	FOR n:=1 UPTO nBoxes
		SELF:aBoxes[n]:Play()
	NEXT

	SELF:Invalidate()

RETURN


METHOD OnPaint(e AS PaintEventArgs) AS VOID CLASS DemoPictureBox

	LOCAL eGr AS Graphics
	LOCAL n,x AS INT
	
	eGr:=e:Graphics
	
	FOR n:=101-nLines UPTO 100
		SELF:aLines[n]:Draw(eGr)
	NEXT
	
	FOR n:=1 UPTO nBoxes
		SELF:aBoxes[n]:Draw(eGr)
	NEXT
		
	SELF:DrawText(eGr)
	SELF:DrawWave(eGr)	
	SELF:DrawBitmap(eGr)


RETURN


METHOD DrawText(eGr AS Graphics) AS VOID CLASS DemoPictureBox

	LOCAL oBrush AS Brush
	LOCAL oCol1,oCol2 AS Color

	IF nCounter>5000
		RETURN
	END IF
	
	oCol1:=Color.White
	oCol2:=Color.Red
	IF nCounter<1500
		oCol2:=Color.FromArgb(Min(nCounter / 5,255),255,0,0)
	ELSEIF nCounter<2000
		oCol2:=Color.FromArgb(255,255,Min(255,(nCounter-1500)/2),Min(255,(nCounter-1500) / 2))
	END IF
	
	DO CASE
	CASE nCounter<2000
		oBrush:=LinearGradientBrush{Point{INT(Sin(nCounter/40.0)*100.0),0},;
								Point{INT(Cos(nCounter/70.0)*100.0)+400, INT(Sin(nCounter/30.0)*250.0)},;
								oCol1,oCol2}
	CASE nCounter<2200
		    oBrush:=SolidBrush{Color.White}
	CASE nCounter<3000
		oBrush:=SolidBrush{Color.FromArgb(Max(0,255-(nCounter-2200)/3),255,255,255)}
	END CASE
	
	IF nCounter < 3000
		eGr:DrawString('Vulcan.Net GDI+',oFont,oBrush,80.0,300.0)
	END IF
	
RETURN

METHOD DrawWave(oGr AS Graphics) AS VOID CLASS DemoPictureBox

	LOCAL oBrush AS Brush
	LOCAL oContainer AS GraphicsContainer
	LOCAL x,y,xc,yc AS REAL4
	LOCAL oSizeF AS SizeF
	LOCAL cPhrase,cLetter AS STRING
	LOCAL n AS INT

	IF SELF:nCounter < 2300 .or. SELF:nCounter > 3500
		RETURN
	END IF
			
	oBrush:=LinearGradientBrush{Point{0,0},Point{40,40},;
				Color.FromArgb(192,Color.Yellow),Color.FromArgb(192,Color.Blue)}
	
	cPhrase := 'Hope you have enjoyed this little GDI+ demo ...'

	x:=800 - (SELF:nCounter - 2300) * 3

	FOR n:=1 UPTO SLen(cPhrase)
	
		y:=100.0 + 40.0 * Sin(x / 70)

		cLetter:=CharPos(cPhrase,n)
		oSizeF:=oGr:MeasureString(cLetter,SELF:oWaveFont)
		
		xc:=x+oSizeF:Width / 2
		yc:=y+oSizeF:Height / 2

		IF x>-50.0 .and. x<850.0
				
			oContainer:=oGr:BeginContainer()
		
			oGr:TranslateTransform(xc,yc)
			oGr:RotateTransform(REAL4(45.0*Cos(-x / 70)))
			oGr:TranslateTransform(-xc,-yc)
			oGr:ScaleTransform( REAL4(1.0 + Sin(nCounter / 60.0) / 3.0) , 1.0 )
			
			oGr:DrawString(cLetter,SELF:oWaveFont,oBrush,x,y)
		
			oGr:EndContainer(oContainer)

		END IF
		
		x+= oSizeF:Width * 0.8
	
	NEXT
	
RETURN


METHOD DrawBitmap(eGr AS Graphics) AS VOID CLASS DemoPictureBox
	LOCAL nStep AS INT
	LOCAL oBrush AS Brush
	
	IF SELF:nCounter > 3400
		
		nStep := SELF:nCounter - 3400
		
		eGr:DrawImage ( SELF:oBitmap , 400 - nStep , 300 - nStep , nStep * 2 , nStep * 2 )

		IF nStep > 450
			((Form)SELF:Parent):Opacity-=0.003
		END IF
		
		IF nStep > 800
			Application.Exit()
		END IF
	
	END IF

	IF nCounter > 3300 .and. SELF:lF1Pressed
		IF (SELF:nCounter / 25) % 2 == 0
			oBrush:=SolidBrush{Color.Red}
			eGr:DrawString('Help file not installed!', Font{'Courier New' , 30.0}, oBrush, 100.0, 500.0)
		END IF
	END IF

	IF nCounter > 3700
		oBrush:=SolidBrush{Color.Red}
		eGr:DrawString('cpc/d', Font{'Courier New' , 9.0}, oBrush, 750.0, 580.0)
	END IF

	
RETURN


CLASS Box

	EXPORT x,y,nSpeed,nXspd,nSize AS INT
	PROTECT oPen AS Pen
	PROTECT oBrush AS Brush

CONSTRUCTOR() CLASS Box

	LOCAL nColor AS INT

	x:=Rnd(500)
	y:=Rnd(300)
	nSpeed:=Rnd(10)
	nSpeed:=0
	nXspd:=Rnd(5)
	nColor:=Rnd(5)
	nSize:=Rnd(20)+20
	DO CASE
	CASE nColor==0
		oPen:=Pen{Color.Gray}
		oBrush:=SolidBrush{Color.Green}
	CASE nColor==1
		oPen:=Pen{Color.Red}
		oBrush:=SolidBrush{Color.White}
	CASE nColor==2
		oPen:=Pen{Color.Blue}
		oBrush:=SolidBrush{Color.Gray}
	CASE nColor==3
		oPen:=Pen{Color.Blue}
		oBrush:=SolidBrush{Color.Red}
	CASE nColor==4
		oPen:=Pen{Color.Yellow}
		oBrush:=SolidBrush{Color.Blue}
	END CASE
RETURN

METHOD Play() AS VOID CLASS Box

	IF nSpeed>0 .and. SELF:y + nSpeed > 300 .and. SELF:y + nSpeed < 370 .and. (x>80 .and. x<720)
		nSpeed:=-nSpeed
	ELSEIF SELF:y + nSpeed > 600
		nSpeed:=-nSpeed
	ELSEIF nSpeed<0 .and. SELF:y + nSpeed < 370 .and. SELF:y + nSpeed > 300 .and. (x>80 .and. x<720)
		nSpeed:=-nSpeed
	ELSE
		nSpeed++
	END IF
	SELF:y+=nSpeed
	
	x+=nXspd
	IF x<0 .or. x>800
		nXspd:=-nXspd
	END IF

RETURN


METHOD Draw(eGr AS Graphics) AS VOID CLASS Box
//eGr:FillRectangle(SolidBrush{Color.Blue},x,y,20,20)
	eGr:FillEllipse(SELF:oBrush,x,y,nSize,nSize)
	eGr:DrawEllipse(SELF:oPen,x,y,nSize,nSize)
/*eGr:DrawLine(Self:oPen,x,y , x+20,y)
eGr:DrawLine(Self:oPen,x+20,y , x+20,y+20)
eGr:DrawLine(Self:oPen,x+20,y+20 , x,y+20)
eGr:DrawLine(Self:oPen,x,y+20 , x,y)*/
RETURN


CLASS Line

	PROTECT x1,x2,y1,y2 AS INT
	PROTECT nIndex AS INT
	PROTECT oPen AS Pen

CONSTRUCTOR(n AS INT) CLASS Line

	nIndex:=n
	n:=n % 3
	DO CASE
	CASE n==0
		oPen:=Pen{Color.FromArgb(160,255,0,0)}
	CASE n==1
		oPen:=Pen{Color.FromArgb(160,255,255,0)}
	CASE n==2
		oPen:=Pen{Color.FromArgb(160,0,0,255)}
	END CASE
	oPen:Width:=4

RETURN

METHOD Play(nCounter AS INT) AS VOID CLASS Line

	IF nCounter > 3800
		RETURN
	END IF
	
	IF nCounter > 2300 .and. nCounter % 6 == 0
		IF SELF:oPen:Color:A > 80
			SELF:oPen:Color := Color.FromArgb(SELF:oPen:Color:A-1 , SELF:oPen:Color)
		END IF
	END IF

	nCounter+=nIndex*3
	x1:=Sin(nCounter/50.0)*400.0 + 400
	y1:=(Cos(nCounter/50.0) + Sin(nCounter/80.0)  )*150.0 + 300
	x2:=Cos(nCounter/100.0)*400.0 + 400
	y2:=Sin(nCounter/50.0)*300.0 + 300

RETURN

METHOD Draw(eGr AS Graphics) AS VOID CLASS Line

	eGr:DrawLine(SELF:oPen,x1,y1,x2,y2)

RETURN


FUNCTION Rnd(nMax AS INT) AS INT

RETURN INT(Microsoft.VisualBasic.VBMath.Rnd()*nMax)

